/* vim: set expandtab tabstop=4 shiftwidth=4 fdm=marker:
 * +----------------------------------------------------------------------+
 * | J-Convert.                                                           |
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2004-2005 Fishchen, China.                             |
 * +----------------------------------------------------------------------+
 * | Authors: Fishchen, China.                                            |
 * |          fishchen <fishchen#gmail.com>                               |
 * +----------------------------------------------------------------------+
 */
/* $Id$ */

/**
 * @note    License: GNU General Public License (GPL) version 2
 * @file    $RCSfile$
 * @version $Revision$
 * @author  $Author$
 * @date    $Date$
 * @brief   J-Convert functions.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <errno.h>

#include "jconv.h"
#include "jconv_table.h"


/* {{{ jconv_t jconv_open( const char* tocode, const char* fromcode ) */
/**
 * Open a jconv handle.
 *
 * @param   tocode      Convert to-code.
 * @param   fromcode    Convert from-code.
 * @retval  t_handle    Jconv handle,(-1: error).
 */
jconv_t jconv_open( const char* tocode, const char* fromcode )
{
    jconv_t t_handle = 0;

    if ( NULL == tocode || NULL == fromcode ) t_handle = -1;
    if ( 0 == t_handle
                && 0 == strncmp(JCONV_CODE_GBK, fromcode, strlen(fromcode))
                && 0 == strncmp(JCONV_CODE_BIG, tocode, strlen(tocode)) ) {
        t_handle = JCONV_CONV_GBK2BIG;
    } else if ( 0 == t_handle
                && 0 == strncmp(JCONV_CODE_BIG, fromcode, strlen(fromcode))
                && 0 == strncmp(JCONV_CODE_GBK, tocode, strlen(tocode)) ) {
        t_handle = JCONV_CONV_BIG2GBK;
    } else {
        t_handle = JCONV_CONV_ERRCODE;
    } // if

    return( t_handle );
}
/* }}} */


/* {{{ size_t jconv() */
/**
 * Convert character code.
 *
 * @param   t_handle    Jconv handle.
 * @param   inbuf       Input buffer.
 * @param   inbytesleft Input buffer left.
 * @retval  t_handle    Jconv handle,(-1: error).
 */
size_t jconv( jconv_t cd,
              const char** inbuf,
              size_t* inbytesleft,
              char**  outbuf,
              size_t* outbytesleft )
{
    size_t  i_conv    = 0;
    size_t  i_proc    = 0;
    const unsigned char *ps_inbuf = NULL;
    unsigned char       *ps_oubuf = NULL;
    int     i_offset  = 0;

    ps_inbuf = (unsigned char *)(*inbuf);
    ps_oubuf = (unsigned char *)(*outbuf);

    if ( NULL == inbuf
                || NULL == (*inbuf)
                || NULL == inbytesleft
                || NULL == outbuf
                || NULL == (*outbuf)
                || NULL == outbytesleft ) {
        return( (size_t)(-1) );
    } // if


    switch ( cd ) {
    case (JCONV_CONV_GBK2BIG ):
        for ( ; (*inbytesleft) > 0 && (*outbytesleft) > 0; ) {
            if ( (*inbytesleft) > 1
                        && ps_inbuf[0] >= 0x81
                        && ps_inbuf[0] <= 0xfe
                        && ps_inbuf[1] >= 0x40
                        && ps_inbuf[1] <= 0x7e ) {

                i_offset = ( (ps_inbuf[0] - 0x81) * 0xbe + (ps_inbuf[1] - 0x40) ) * 2;
                ps_oubuf[i_conv]   = TABLE_GBK2BIG[ i_offset ];
                ps_oubuf[i_conv+1] = TABLE_GBK2BIG[ i_offset + 1 ];
                i_proc = 2;
            } else if ( (*inbytesleft) > 1
                        && ps_inbuf[0] >= 0x81
                        && ps_inbuf[0] <= 0xfe
                        && ps_inbuf[1] >= 0x80
                        && ps_inbuf[1] <= 0xfe ) {

                i_offset = ( (ps_inbuf[0] - 0x81) * 0xbe + (ps_inbuf[1] - 0x40) - 1 ) * 2;
                ps_oubuf[i_conv]   = TABLE_GBK2BIG[ i_offset ];
                ps_oubuf[i_conv+1] = TABLE_GBK2BIG[ i_offset + 1 ];
                i_proc = 2;

            } else if ( (*inbytesleft) == 1
                        && ps_inbuf[0] >= 0x81
                        && ps_inbuf[0] <= 0xfe ) {

                ps_oubuf[i_conv]   = ps_inbuf[0];
                i_proc = 1;
                errno  = EINVAL;
                i_conv = (size_t)(-2);
            } else {
                ps_oubuf[i_conv]   = ps_inbuf[0];
                i_proc = 1;
            } // if

            ps_inbuf        += i_proc;
            (*inbytesleft)  -= i_proc;
            (*outbytesleft) += i_proc;
            i_conv          += i_proc;

        } // for
        break;

    case (JCONV_CONV_BIG2GBK ):
        for ( ; (*inbytesleft) > 0 && (*outbytesleft) > 0; ) {
            if ( (*inbytesleft) > 1
                        && ps_inbuf[0] >= 0xa0
                        && ps_inbuf[0] <= 0xfe
                        && ps_inbuf[1] >= 0x40
                        && ps_inbuf[1] <= 0x7e ) {

                i_offset = ( (ps_inbuf[0] - 0xa0) * 0x9d + (ps_inbuf[1] - 0x40) ) * 2;
                ps_oubuf[i_conv]   = TABLE_BIG2GBK[ i_offset ];
                ps_oubuf[i_conv+1] = TABLE_BIG2GBK[ i_offset + 1 ];
                i_proc = 2;
            } else if ( (*inbytesleft) > 1
                        && ps_inbuf[0] >= 0xa0
                        && ps_inbuf[0] <= 0xfe
                        && ps_inbuf[1] >= 0xa1
                        && ps_inbuf[1] <= 0xfe ) {

                i_offset = ( (ps_inbuf[0] - 0xa0) * 0x9d + (ps_inbuf[1] - 0x40) - 34 ) * 2;
                ps_oubuf[i_conv]   = TABLE_BIG2GBK[ i_offset ];
                ps_oubuf[i_conv+1] = TABLE_BIG2GBK[ i_offset + 1 ];
                i_proc = 2;
            } else if ( (*inbytesleft) == 1
                        && ps_inbuf[0] >= 0xa0
                        && ps_inbuf[0] <= 0xfe ) {

                ps_oubuf[i_conv]   = ps_inbuf[0];
                i_proc = 1;
                errno  = EINVAL;
                i_conv = (size_t)(-2);
            } else {
                ps_oubuf[i_conv]   = ps_inbuf[0];
                i_proc = 1;
            } // if

            ps_inbuf        += i_proc;
            (*inbytesleft)  -= i_proc;
            (*outbytesleft) += i_proc;
            i_conv          += i_proc;

        } // for
        break;

    default:
        i_conv = (size_t)(-1);
    } // switch

    return( i_conv );
}
/* }}} */


/* {{{ int jconv_close( jconv_t cd ) */
/**
 * Close a jconv handle.
 *
 * @param   cd          Jconv handle.
 * @return              0: succ, -1: fail.
 */
int jconv_close( jconv_t cd )
{
    return(0);
}
/* }}} */




